function z = BILS(A,y,l,u)
%
% z = BILS(A,y,l,u) produces the solution to 
%               the box-constrained integer least squares (BILS) problem
%               min_z||y-Az||^2, where l <= z <= u
%               
% Input arguments:
%    A ---- m by n real matrix with full column rank
%    y ---- m-dimensional real vector
%    l ---- n-dimensional lower-bound integer vector
%    u ---- n-dimensional upper-bound integer vector
% 
% Output argument:
%    z ---- n-dimensional integer vector.  It's the solution to the BILS
%           problem.
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.


% Check input arguments
if nargin < 4 
    error('Missing input arguments')
end

[m,n] = size(A);
if m < n || m ~= size(y,1) || size(y,2) ~= 1 || n ~= size(l,1) || n ~= size(u,1)  
    error('Input arguments have a matrix dimension error')
end

if rank(A) < n 
	error('Matrix is rank defficient')
end

% Reduction phase
[R P yt l u zh] = reduction_bils(A,y,l,u);

% Set ellipsoid bound based on the Babai integer point
B = norm(yt - R*zh)^2;

% Search phase
[z found] = search_bils(R,yt,l,u,B); 

if found ~= 1
z = zh; % if no point is found in the search, then the solution is the Babai integer 
        % point (found in the reduction).  This is the solution of the upper triangular 
        % BILS problem.
end

% To obtain the solution of the general BILS problem, we do
% the equivalent of z = P*z (since we implemented P as a vector).
for i = 1:1:n
    if P(i) ~= 0
       z([i P(i)]) = z([P(i) i]);
       P(P(i)) = 0;
    end
end
