function [R P yt l u zh]=reduction_bils(A,y,l,u)
%
% [R P yt l u zh]=reduction(A,y,l,u) reduces the general box-constraint 
%                 integer least squares problem to an upper triangular one.  
%                 The permutations  of the columns of the upper triangular 
%                 are determined such as to reduce the search range for 
%                 the optimal solution to the BILS problem.
%
% Input arguments:
%    A ---- m by n real matrix with full column rank
%    y ---- m-dimensional real vector
%    l ---- n-dimensional lower-bound integer vector
%    u ---- n-dimensional upper-bound integer vector
% 
% Output arguments:
%    R  ---- n by n real upper triangular matrix
%    P  ---- n-dimensional integer permutation vector 
%    yt ---- n-dimensional real vector.  It's the updated value of y
%    zh ---- n-dimensional integer vector.  It's the Babai integer point
%    l  ---- n-dimensional updated lower-bound integer vector 
%    u  ---- n-dimensional updated upper-bound integer vector 
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.

% Check input arguments
if nargin < 4 
    error('Missing input arguments')
end

[m,n] = size(A);
if m < n || m ~= size(y,1) || size(y,2) ~= 1 || n ~= size(l,1) || n ~= size(u,1) 
    error('Input arguments have a matrix dimension error')
end

if rank(A) < n, 
	error('Matrix is rank defficient')
end

[Q R] = qr(A);
[m n] = size(A);
R = R(1:n,1:n);
yt=Q(1:m,1:n)'*y;

P = zeros(n,1);
zh= zeros(n,1);
yh = yt;

for k=n:-1:2
	if k < n
        % To compute ck and later cj (j < k)
		yh(1:k)= yh(1:k) - R(1:k,k+1)*zh(k+1);
    end

    ck= yh(k)/R(k,k);
    % Set zk to the 2nd nearest integer on [l(k), u(k)] to ck
    zk = nearInterval(l(k),u(k),ck,2);
    a = abs(R(k,k)*(zk - ck));  
    p=k;
    Rtmp = R(1:k,1:k);
    ytmp = yh(1:k);

    for j=1:k-1
        Rp = Rtmp;
        yp = ytmp;
        % Interchange columns j and k of Rp and transform it to an upper
        % triangular matrix by Givens rotations
        [Gt Rp]=ReducGiv(Rp,j,k);
        % Update yp
        yp=Gt*yp;
        cpk = yp(k)/Rp(k,k);
	% TODO change ks to js in below line
        zpk = nearInterval(l(j), u(j),cpk,2);
        ap = abs(Rp(k,k)*(zpk-cpk));

        if ap > a        
            
            % |Rp(k,k)*(zp(k) - cp(k))| is the largest value obtained, therefore we
            % keep this column interchange. 
            a = ap;
            p = j;
            R(1:k,1:k)=Rp;
            yh(1:k) = yp;
            ck = cpk;
            % Store the Givens rotation
            storeGt = Gt;
          end
    end   

    if p ~= k
        % A Column interchange occured.  Update the values of P,l,u,yt, and R.
        P(k)=p;
        P(p) = k;
	% TODO decomment below lines
        l([p k])=l([k p]);
        u([p k])= u([k p]);
        yt(1:k) = storeGt * yt(1:k);
        R(1:k,k+1:n)= storeGt * R(1:k,k+1:n); 
    end
    % Set zk to the first nearest integer on [l(k), u(k)] to ck
    zh(k) = nearInterval(l(k),u(k),ck,1);
end

yh(1) = yh(1) - R(1,2)*zh(2);
c1 = yh(1)/R(1,1);
zh(1) = nearInterval(l(1),u(1),c1,1);



