function [zh found]=search_bils(R,y,l,u,B)
%
% zh = search(R,y,l,u,B) produces the solution to the upper triangular
%       box-constraint integer least squares problem min_{z}||y-Rz|^2 (with l<=z<=u)
%       by a search algorithm.
%
% Input arguments:
%    R ---- n by n real nonsingular upper triangular matrix
%    y ---- n-dimensional real vector
%    l ---- n-dimensional lower-bound integer vector
%    u ---- n-dimensional upper-bound integer vector
%    B ---- positive real number.  It's the ellipsoid bound
%
% Output arguments:
%    zh -   n-dimensional integer vector.   It's the solution to the 
%           upper triangular box-constrained integer least squares problem. 
% found -   flag. A value of 1 indicates that the search found a point. Other 
%           values indicates that no point was found. 
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, May 2007.


% Check input arguments
if nargin < 5
    error('Missing input arguments')
end

[m,n] = size(R);

if m ~= n || n ~= size(y,1) || size(y,2) ~= 1 || n ~= size(l,1) || n ~= size(u,1) 
    error('Input arguments have a matrix dimension error')
end

% Initialization
T = zeros(n,1); % T(k) = Sum_i=k+1_to_n { R(i,i)^2 * (z(i) - c(i))^2 } 
c = zeros(n,1); % c(k) = (y(k) - R(k,k+1:n) * z(k+1:n))/R(k,k)
z = zeros(n,1); % Potential solution to upper triangular BILS problem
zh = zeros(n,1);
lbounds = zeros(n,1); % lbounds(k) = 1 if z(k) = l(k) 
ubounds = zeros(n,1); % ubounds(k) = 1 if z(k) = u(k)
delta = zeros(n,1);  % delta(k) is used to enumerate new points at level k
found = 0;

k=n; % k is the current level of the search
T(k) = 0;   

s = compute_sigma(R,y,l,u);

% Compute c(k), z(k), lbounds(k), ubounds(k) and delta(k)
[c(k) z(k) lbounds(k) ubounds(k) delta(k)]=compute_zk(R,y,z,l,u,k);


while 1
        
   if s(k) + T(k) + R(k,k)^2 * (z(k) - c(k))^2 >= B
        % We are not inside the ellipsoid               
        if k == n
           return; 
        else
           k = k + 1;              
        end
       
   elseif k > 1
       % We go to level k-1
       T(k-1) = T(k) + R(k,k)^2 * (z(k) - c(k))^2;  
       k = k-1;
       [c(k) z(k) lbounds(k) ubounds(k) delta(k)]=compute_zk(R,y,z,l,u,k);          
       continue;
   else % k = 1 and a valid point is found 
       B = T(1) + R(1,1)^2 * (z(1) - c(1))^2;  
       zh = z;
       k = k+1;
       found = 1;
  end

   while 1
    
       if ubounds(k) == 1 && lbounds(k) == 1 % no integer is available at this level
            if k == n
               return;
            else
              k = k+1;
            end
        continue;
        end
       
       % Enumeration at level k
       z(k) = z(k) + delta(k);       
       if z(k) == l(k)
           lbounds(k) = 1;
           delta(k) = -delta(k) - sign(delta(k));
       elseif z(k) == u(k)
           ubounds(k) = 1;
           delta(k) = -delta(k) - sign(delta(k));
       elseif lbounds(k) == 1
           delta(k) = 1;
       elseif ubounds(k) == 1
           delta(k) = -1;
       else
           delta(k) = -delta(k) - sign(delta(k));
       end
       break;
   end
 end




