function zh=search_eils(R,y,B,a)
%
% zh = search_eils(R,y,B,a) computes the solution to the upper triangular
% EILS problem by a search algorithm.  
%                            
% Input arguments:
%   R ----  n by n real nonsingular upper triangular matrix 
%   y ----  n-dimensional real vector
%   B ----  positive real number.  It's the ellipsoid bound
%   a ----  real number.  It's the ellipsoid-constraint
%   
% Output argument:
%    zh --- n-dimensional integer vector.  It's the solution to the upper 
%           triangular EILS problem.    
%
% Copyright (c) 2007. Xiao-Wen Chang and Mazen Al Borno
% Version 1.0, June 2007.

% Check input arguments
if nargin < 4
    error('Missing input arguments')
end

[m,n] = size(R);

if m ~= n || n ~= size(y,1) || size(y,2) ~= 1  
    error('Input arguments have a matrix dimension error')
end

% Initialization
T = zeros(n,1);  % T(n)=0 and T(k) = Sum_i=k+1_to_n { R(i,i)^2 * (z(i) - c(i))^2 } 
b = zeros(n,1);  % b(n) = 0 and b(k) = Sum_j=k+1_to_n R(k,j)*z(j)  
S = zeros(n,1);  % S(n) = a^2 and S(k-1) = S(k) - (R(k,k)*z(k) + b(k))^2  
c = zeros(n,1);  % c(k) = (y(k) - R(k,k+1:n) * z(k+1:n))/R(k,k)
z = zeros(n,1);  % Potential solution to upper triangular EILS problem
zh = zeros(n,1);
l = zeros(n,1);  % lower bound integer vector
u = zeros(n,1);  % upper bound integer vector
lbound = zeros(n,1);  % lbounds(k) = 1 if z(k) = l(k)
ubound = zeros(n,1);  % ubounds(k) = 1 if z(k) = u(k)
delta = zeros(n,1);   % delta(k) is used to enumerate new points at level k

k=n;
b(k) = 0;
S(k) = a^2;
T(k) = 0;

l(k) = ceil((-sqrt(S(k))-b(k))/R(k,k));
u(k) = floor((sqrt(S(k)) - b(k))/R(k,k));

if u(k) >= l(k)
    % Compute c(k), z(k), lbound(k), ubound(k) and delta(k)
    [c(k) z(k) lbound(k) ubound(k) delta(k)]=compute_zk_eils(R(k,k),y(k),b(k),l(k),u(k));
end


while 1
     
if T(k) + R(k,k)^2 *(z(k) - c(k))^2 >= B || u(k) < l(k)
           
    % We are not inside the ellipsoid
    if k == n
	return
    else 
        k = k+1;
    end

elseif k > 1
     % We go to level k-1
    b(k-1) = R(k-1,k:n)*z(k:n); 
    T(k-1) = T(k) + R(k,k)^2 *(z(k) - c(k))^2;
    S(k-1) = S(k) - (b(k) + R(k,k)*z(k))^2;
    
    k = k-1;
    l(k) = ceil((-sqrt(S(k))-b(k))/R(k,k));
    u(k) = floor((sqrt(S(k)) - b(k))/R(k,k));
    if u(k) >= l(k)
    [c(k) z(k) lbound(k) ubound(k) delta(k)]=compute_zk_eils(R(k,k),y(k),b(k),l(k),u(k));
    end
    continue;
else 
    % k = 1 and a valid point was found
    B = T(1) + R(1,1)^2 *(z(1) - c(1))^2;
    zh = z;
    k = k + 1;
end

    while 1  % Enumeration at level k

    if ubound(k) == 1 && lbound(k) == 1  % no integer is available at this level
        if k == n
          return
        else 
          k = k + 1;
        end
          continue;
    end
    
    z(k) = z(k) + delta(k); 
    if z(k) == l(k)
        lbound(k) = 1;
        delta(k) = -delta(k) - sign(delta(k));
    elseif z(k) == u(k)
        ubound(k) = 1;
        delta(k) = -delta(k) - sign(delta(k));
    elseif lbound(k) == 1
        delta(k) = 1;
    elseif ubound(k) == 1
        delta(k) = -1;
    else 
        delta(k) = -delta(k) - sign(delta(k));
    end
    break;
    end

end
